"""
Git analysis module for ReviewLlama using functional programming style.
"""

from dataclasses import dataclass
from pathlib import Path
from typing import List, Optional, Tuple

from git import Repo
from git.diff import Diff
from git.exc import GitCommandError, InvalidGitRepositoryError
from git.objects import Commit
from rich.console import Console


@dataclass(frozen=True)
class GitDiff:
    """Represents a git diff with metadata."""

    file_path: str
    old_content: str
    new_content: str
    diff_text: str
    change_type: str  # 'added', 'modified', 'deleted'


@dataclass(frozen=True)
class GitAnalysis:
    """Complete git analysis result."""

    repo: Repo
    repository_path: Path
    current_branch: str
    base_branch: str
    diffs: List[GitDiff]
    total_files_changed: int


def find_git_repository(path: Path) -> Repo:
    """Find and return git repository from given path."""
    try:
        return Repo(path, search_parent_directories=True)
    except InvalidGitRepositoryError:
        raise ValueError(f"No git repository found at or above {path}")


def get_current_branch_name(repo: Repo) -> str:
    """Get the name of the current branch."""
    try:
        return repo.active_branch.name
    except TypeError:
        # Detached HEAD state
        return repo.head.commit.hexsha[:8]


def branch_exists(repo: Repo, branch_name: str) -> bool:
    """Check if a branch exists in the repository."""
    try:
        repo.commit(branch_name)
        return True
    except:
        return False


def get_tracked_files(repo: Repo):
    return [
        entry.abspath
        for entry in repo.commit().tree.traverse()
        if Path(entry.abspath).is_file()
    ]


def get_base_branch(repo: Repo, requested_base: str) -> str:
    """Determine the base branch to compare against."""
    # Try requested base first
    if branch_exists(repo, requested_base):
        return requested_base

    # Fall back to common master branch names
    common_masters = ["master", "main", "develop"]
    for branch in common_masters:
        if branch_exists(repo, branch):
            return branch

    raise ValueError(
        f"Base branch '{requested_base}' not found and no common master branch exists"
    )


def get_diff_between_branches(repo: Repo, base_branch: str, current_branch: str):
    """Get diff between two branches."""
    try:
        base_commit = repo.commit(base_branch)
        current_commit = repo.commit(current_branch)
        return base_commit.diff(current_commit)
    except GitCommandError as e:
        raise ValueError(
            f"Failed to get diff between {base_branch} and {current_branch}: {e}"
        )


def determine_change_type(diff_item: Diff) -> str:
    """Determine the type of change from a diff item."""
    if diff_item.new_file:
        return "added"
    elif diff_item.deleted_file:
        return "deleted"
    else:
        return "modified"


def extract_file_content(diff_item: Diff, is_old: bool = True) -> str:
    """Extract file content from diff item."""
    try:
        blob = diff_item.a_blob if is_old else diff_item.b_blob
        if blob is None:
            return ""
        return blob.data_stream.read().decode("utf-8", errors="ignore")
    except (UnicodeDecodeError, AttributeError):
        return ""


def create_git_diff(diff_item: Diff) -> GitDiff:
    """Create GitDiff from git.Diff object."""
    file_path = diff_item.a_path or diff_item.b_path or "unknown"
    old_content = extract_file_content(diff_item, is_old=True)
    new_content = extract_file_content(diff_item, is_old=False)
    diff_text = str(diff_item)
    change_type = determine_change_type(diff_item)

    return GitDiff(
        file_path=file_path,
        old_content=old_content,
        new_content=new_content,
        diff_text=diff_text,
        change_type=change_type,
    )


def process_diff_items(diff_index) -> List[GitDiff]:
    """Process all diff items into GitDiff objects."""
    return [create_git_diff(item) for item in diff_index]


def filter_reviewable_diffs(diffs: List[GitDiff]) -> List[GitDiff]:
    """Filter diffs to only include reviewable files."""

    # TODO: Update this to a more complete list of programming language extensions
    reviewable_extensions = {
        ".py",
        ".js",
        ".ts",
        ".java",
        ".cpp",
        ".c",
        ".h",
        ".hpp",
        ".go",
        ".rs",
        ".rb",
        ".php",
        ".cs",
        ".swift",
        ".kt",
        ".scala",
        ".clj",
        ".sh",
        ".sql",
        ".yml",
        ".yaml",
        ".json",
    }

    def is_reviewable(diff: GitDiff) -> bool:
        path = Path(diff.file_path)
        return path.suffix.lower() in reviewable_extensions

    return [diff for diff in diffs if is_reviewable(diff)]


def analyze_git_repository(path: Path, base_branch: str) -> GitAnalysis:
    """Analyze git repository and extract diffs for review."""
    repo = find_git_repository(path)
    current_branch = get_current_branch_name(repo)
    resolved_base_branch = get_base_branch(repo, base_branch)

    diff_index = get_diff_between_branches(repo, resolved_base_branch, current_branch)
    all_diffs = process_diff_items(diff_index)
    reviewable_diffs = filter_reviewable_diffs(all_diffs)

    return GitAnalysis(
        repo=repo,
        repository_path=path,
        current_branch=current_branch,
        base_branch=resolved_base_branch,
        diffs=reviewable_diffs,
        total_files_changed=len(reviewable_diffs),
    )
